/*
**  Estimation of a Vector ARMA(1,1) process
**  Conditional Maximum Likelihood
**
**  Reinsel, G.C. [1993], Elements of Multivariate Time Series Analysis,
**  Springer-Verlag, New York
*/

new;
library tsm,optmum;

output file = tsm1b.out reset;

@<--- data --->@

load reinsel[100,2] = reinsel.asc;

invest = reinsel[.,1];
invent = reinsel[.,2];
di = invest - lag1(invest);
y = di~invent;

_print = 1;
_tsm_optmum = 0;   /* Analytical Newton-Raphson algorithm */
_tsm_gtol = 0.001;


@<--- Unrestricted model --->@

sv = "HR2"; /* Hannan-Rissanen method (second form) */

{beta1,stderr1,Mcov1,LogL1} = arma_ML(y,1,1,"HR2");
SIGMA1 = _arma_SIGMA;

save arma1 = beta1;
save arma2 = sigma1;


@<--- Restricted model (implicit form) --->@

/*
**  4 restrictions:
**
**  AR1_21 = 0
**  AR1_12 = 0
**  MA1_21 = 0
**  MA1_12 = 0
*/

RR = design(1|0|0|2|3|0|0|4); r = zeros(8,1);

sv = 0; /* Lutkepohl method */

{beta2,stderr2,Mcov2,LogL2} = arma_CML(y,1,1,sv,RR,r);
SIGMA2 = _arma_SIGMA;


@<--- Testing restrictions --->@

H = vread(_ml_derivatives,"H_matrix");  /* Hessian matrix (restricted model) */
G = vread(_ml_derivatives,"G_matrix");  /* Gradient matrix (restricted model) */

I = - H;                                /* Approximation of the Information matrix */


/*
**  Likelihood ratio statistic
**
**  DAVIDSON and MACKINNON [1993], Estimation and Inference in Econometrics
**  Oxford University Press, page 437, formula (13.06)
*/


LR = 2*(logl1-logl2);      /* Likelihood ratio                             */
pvalue = cdfchic(LR,4);    /* Approximating the noncentral chi-squared CDF */

print;
print ftos(LR,"Likelihood ratio statistic: %lf",10,5);
print ftos(pvalue,"p-value: %lf",10,5);


/*
**  Lagrange multiplier
**
**  DAVIDSON and MACKINNON [1993], Estimation and Inference in Econometrics
**  Oxford University Press, page 437, formula (13.04)
*/

LM = G'*inv(I)*G;
pvalue = cdfchic(LM,4);

print;
print ftos(LM,"Lagrange multiplier: %lf",10,5);
print ftos(pvalue,"p-value: %lf",10,5);


/*
**  Wald test
**
**  DAVIDSON and MACKINNON [1993], Estimation and Inference in Econometrics
**  Oxford University Press, page 437, formula (13.05)
*/

proc restrict(beta);
  local r;
  r = beta[2]|beta[3]|beta[6]|beta[7];
  retp(r);
endp;

Dr = gradp(&restrict,beta1);
r = restrict(beta1);

wald = r'*invpd(Dr*Mcov1*Dr')*r;
pvalue = cdfchic(wald,4);

print;
print ftos(wald,"Wald statistic: %lf",10,5);
print ftos(pvalue,"p-value: %lf",10,5);
print; print;


@<--- Responses to Forecast Errors (unrestricted model) --->@

call arma_impulse(beta1,1,1,8);

z = miss(zeros(2,1),0);

mask = ones(1,5);
let fmt[5,3]=
    "*.*lf" 10 3
    "*.*lf" 10 3
    "*.*lf" 25 0
    "*.*lf" 10 3
    "*.*lf" 10 3;

i = 0;
do until i>8;
  x = varget("IMPULSE"$+ftos(i,"%lf",1,0));
  y = varget("_IMPULSE"$+ftos(i,"%lf",1,0));
  w = x~z~y;

  print chrs(45*ones(79,1));
  print ftos(i,"Periods: %lf",1,0);
  print chrs(45*ones(79,1));
  print "        Estimated responses                   Accumulated responses";
  print "            PHI matrix                             PSI matrix      ";
  print chrs(45*ones(79,1));
  call printfm(w,mask,fmt);
  print;
  i = i+1;
endo;

print; print;


@<--- Responses to Orthogonal Impulses (unrestricted model) --->@

let fmt[5,3]=
    "*.*le" 10 2
    "*.*le" 10 2
    "*.*lf" 25 0
    "*.*le" 10 2
    "*.*le" 10 2;

call arma_orthogonal(beta1,1,1,SIGMA1,8);

i = 0;
do until i>8;
  x = varget("IMPULSE"$+ftos(i,"%lf",1,0));
  y = varget("_IMPULSE"$+ftos(i,"%lf",1,0));
  w = x~z~y;

  print chrs(45*ones(79,1));
  print ftos(i,"Periods: %lf",1,0);
  print chrs(45*ones(79,1));
  print "        Estimated responses                   Accumulated responses";
  print "           THETA matrix                            KSI matrix      ";
  print chrs(45*ones(79,1));
  call printfm(w,mask,fmt);
  print;
  i = i+1;
endo;


output off;

