/*
**  HARVEY [1990], Forecasting, Structural Time Series and
**  the Kalman Filter, Cambridge University Press, pages 86-89
**
**  Cycle Model
*/

new;
library tsm,optmum,pgraph;


@<--- data --->@

load rainfall[] = rainfall.asc;

y = rainfall[1:131];
y = y - 142.2;

_cycle_prmt = 0;
_tsm_parnm = "rho"|"lambda_c"|"sig_kappa"|"sig_epsilon";
_tsm_optmum = 1;
_print = 1;


@<--- Noisy cycle model estimation --->@

/*
**  Spectral generating function for a stochastic cycle plus noise model
*/

proc sgf(theta,lambda);
  local g_cycle,g_noise,g;
  g_cycle = _cycle_sgf(theta[1:3],lambda);
  g_noise = theta[4]^2;
  g = g_cycle + g_noise;
  retp(g);
endp;


/*
**  Using Harvey's starting values, page 198
*/

rho = 0.5; lambda_c = 0.78; sig_epsilon = 1; sig_kappa = 1;
sv = rho|lambda_c|sig_epsilon|sig_kappa;

{theta,stderr,Mcov,Logl} =  FD_ml(y,&sgf,sv);


@<--- Periodogram and spectral function estimation --->@

{lambda,I} = PDGM(y);
_smoothing = 6|5|0|0.23;   /* Parzen lag window with bandwidth = 6   */
I = smoothing(I);          /* smoothed periodogram                   */
g = sgf(theta,lambda);     /* estimated spectral generating function */
q = trunc(rows(lambda)/2);


/*
**  PAWITAN and O'SULLIVAN [1994], Nonparametric spectral density estimation
**  using penalized whittle likelihood,
**  Journal of the American Statistical Association, pages 600-610
*/

/*
**  I: periodogram
**  f: spectral density function
**
**  The autors use the fact that 2I/f is asymptotically
**  distributed as a chi-squared function with 2 degrees of freedom
**  to plot the observed quantiles of 2I/f against
**  the quantiles of the chi(2) distribution.
*/

z = 2*I./g;

x1 = sortc(z,1); Nobs = rows(x1);  y1 = seqa(1,1,Nobs)/Nobs;
x2 = seqa(0,10/Nobs,Nobs);         y2 = 1 - cdfchic(x2,2);


graphset;
  begwind;
  window2(0,2,1,0.75);

  setwind(1);

    _pnum = 0; _paxes = 0; _ptitlht = 1.5;
    title("Rainfall data - Noisy cycle model estimation");
    draw;

  graphset;

  setwind(2);

    _pdate = ""; _pnum = 2; _pnumht = 0.20; _paxht = 0.25;
    fonts("simplex simgrma");
    _plegstr = "Periodogram of the Rainfall data"\
             "\000Estimated spectral generating function";
    _plegctl = {2 5 2.5 4};
    xlabel("frequency");
    xtics(0,pi,pi/4,0);
    lab = " 0 \202p\201/4 \202p\201/2 \2013\202p\201/4 \202p\201";
    asclabel(lab,0);
    xy(lambda[1:q],I[1:q]~g[1:q]);

  setwind(3);

    _plegstr = "\202h\201[2](2)\000empirical distribution of 2I/f";
    _plegctl = {2 6 2.5 2};
    let w = {.,.};
    scale(0|10,w);
    asclabel(0,0);
    xlabel("");
    xy(x2~x1,y2~y1);

    graphprt("-c=1 -cf=tsm1f.eps");
  endwind;