/*
**  Time-Varying Model
**
**  y(t) = b0*x0(t) + b1*x1(t) + u(t)
**  b0(t) = b0(t-1) + v0(t)
**  b1(t) = b1(t-1) + v1(t)
**
**  Maximum Likelihood Estimation
*/

new;
library tsm,optmum,pgraph;
TSMset;

declare external sigma;


@<--- Time-Varying Model simulation --->@

rndseed 123;

s = seqa(1,1,100);

b0 = recserar(rndn(100,1)*1.5,10,1);
b1 = recserar(rndn(100,1)*0.2,4,1);

x0 = ones(100,1);
x1 = rndu(100,1)*25;

u = rndn(100,1)*2;
X = x0~x1;
Y = X0.*b0 + X1.*b1 + u;


@<--- Associated state-space model --->@

proc Z(i); local w; w = X[i,.]; retp(w); endp;

proc d(i); local w; w = 0; retp(w); endp;

proc T(i); local w; w = eye(2); retp(w); endp;

proc c(i); local w; w = 0|0; retp(w); endp;

proc R(i); local w; w = eye(2); retp(w); endp;

proc H(i); local w; w = sigma[1]^2; retp(w); endp;

proc Q(i); local w; w = eye(2).*sigma[2:3]'; w = w^2; retp(w); endp;


@<--- log-likelihood function --->@

proc ml(theta);
  local a0,P0,LogL;
  sigma = theta[1:3];
  call SSM_build(&Z,&d,&H,&T,&c,&R,&Q,1);
  a0 = b0[1]|b1[1];
  P0 = zeros(2,2);
  call KFiltering(Y,a0,P0);
  LogL = KF_ml;
  retp(LogL);
endp;


@<--- ML estimation --->@

_tsm_optmum = 0;                         /* BHHH algorithm */
_tsm_parnm = "sig_u"|"sigma0"|"sigma1";

{theta,stderr,Mcov,LogL} = TD_ml(&ml,2|1.5|0.2);

beta = invpd(X'X)*X'Y;                   /* OLS estimation */



@<--- Kalman filtering --->@

sigma = theta[1:3];
a0 = b0[1]|b1[1];
P0 = zeros(2,2);

call SSM_build(&Z,&d,&H,&T,&c,&R,&Q,1);
call KFiltering(Y,a0,P0);

yc = KF_matrix(1);     /* y[t|t-1]      */
v = KF_matrix(2);      /* v[t]          */
a = KF_matrix(3);      /* a[t]          */
P = KF_matrix(4);      /* P[t]          */
ac = KF_matrix(5);     /* a[t|t-1]      */
Pc = KF_matrix(6);     /* P[t|t-1]      */
F = KF_matrix(7);      /* F[t|t-1]      */
invF = KF_matrix(8);   /* F[t|t-1]^(-1) */


@<--- Kalman smoothing --->@

{as,Ps} = Ksmoothing;


graphset;
   begwind;
   window(1,2,1);

   _pdate = ""; _pnum = 2; _paxht = 0.25; _pnumht = 0.25;
   fonts("simplex simgrma"); 
   xlabel("t");
   _pltype = 6|1|3|4;
   _plegctl = {2 5 4 1};

   setwind(1);
   
     _plegstr = "\202b\201]0[ (true values)"\
                "\000Kalman filter"\
                "\000Kalman smoothing"\
                "\000OLS";
     xy(s,b0~a[.,1]~as[.,1]~(beta[1]*ones(100,1)));

   setwind(2);

     _plegstr = "\202b\201]1[ (true values)"\
                "\000Kalman filter"\
                "\000Kalman smoothing"\
                "\000OLS";
     xy(s,b1~a[.,2]~as[.,2]~(beta[2]*ones(100,1)));

   graphprt("-c=1 -cf=tsm3c.eps");

   endwind;
